#include "EXTERN.h"
#include "perl.h"
#include "XSUB.h"

#include "ppport.h"
#include <sys/epoll.h>
#include <sys/syscall.h>
#include <math.h>
#include <errno.h>

#define __NR_epoll_create	254
#define __NR_epoll_ctl      255
#define __NR_epoll_wait		256
#define SYS_epoll_create	__NR_epoll_create
#define SYS_epoll_ctl		__NR_epoll_ctl
#define SYS_epoll_wait		__NR_epoll_wait

#define MAXEVENTS 1000
extern int errno; 
static struct epoll_event events[MAXEVENTS];

MODULE = POWER::Epoll		PACKAGE = POWER::Epoll		

int
epoll_create(size)
    int size;
    CODE: 
	RETVAL = syscall(__NR_epoll_create, size);
	if (RETVAL == -1)
	    XSRETURN_UNDEF;
    OUTPUT:
	RETVAL
	
int 
epoll_add(epfd, fd, events)
    int epfd;
    int fd;
    int events;
    INIT: 
	struct epoll_event ev;
    CODE: 
	ev.data.fd = fd;
	ev.events  = events;
	RETVAL = 1 + syscall(__NR_epoll_ctl, epfd, EPOLL_CTL_ADD, fd, &ev);
	if (RETVAL == 0)
	    XSRETURN_UNDEF;
    OUTPUT:
	RETVAL

int 
epoll_mod(epfd, fd, events)
    int epfd;
    int fd;
    int events;
    INIT: 
	struct epoll_event ev;
    CODE: 
	ev.data.fd = fd;
	ev.events  = events;
	RETVAL = 1 + syscall(__NR_epoll_ctl, epfd, EPOLL_CTL_MOD, fd, &ev);
	if (RETVAL == 0)
	    XSRETURN_UNDEF;
    OUTPUT:
	RETVAL

int 
epoll_del(epfd, fd)
    int epfd;
    int fd;
    INIT: 
	struct epoll_event ev;
    CODE: 
	ev.data.fd = fd;
	ev.events  = 0;
	RETVAL = 1 + syscall(__NR_epoll_ctl, epfd, EPOLL_CTL_DEL, fd, &ev);
	if (RETVAL == 0)
	    XSRETURN_UNDEF;
    OUTPUT:
	RETVAL

void 
epoll_wait(epfd, timeout)
    int epfd;
    SV * timeout;
    INIT:
	int n, nfds, timeout_ms;
	AV * results;

	timeout_ms = (SvOK(timeout)) ? (int)(SvNV(timeout) * 1000) : -1;
	results = (AV *)sv_2mortal((SV *)newAV());
    PPCODE:
	// TODO timeout = -1 if not defined timeout
	nfds = syscall(__NR_epoll_wait, epfd, &events, MAXEVENTS, timeout_ms);
	XPUSHs(sv_2mortal(newSViv(nfds)));
	av_extend(results, nfds); // optimization
	for(n = 0; n < nfds; ++n) {
	    AV * av;
	    av = (AV *)sv_2mortal((SV *)newAV());
	    av_push(av, newSViv(events[n].data.fd));
	    av_push(av, newSViv(events[n].events));
	    av_push(results, newRV((SV *)av));
	}
	XPUSHs(sv_2mortal(newRV((SV *)results)));

int
EPOLLIN()
    CODE:
	RETVAL = EPOLLIN;
    OUTPUT:
	RETVAL

int
EPOLLOUT()
    CODE:
	RETVAL = EPOLLOUT;
    OUTPUT:
	RETVAL

int
EPOLLPRI()
    CODE:
	RETVAL = EPOLLPRI;
    OUTPUT:
	RETVAL

int
EPOLLERR()
    CODE:
	RETVAL = EPOLLERR;
    OUTPUT:
	RETVAL

int
EPOLLHUP()
    CODE:
	RETVAL = EPOLLHUP;
    OUTPUT:
	RETVAL

int
EPOLLET()
    CODE:
	RETVAL = EPOLLET;
    OUTPUT:
	RETVAL

int
EPOLLONESHOT()
    CODE:
	RETVAL = EPOLLONESHOT;
    OUTPUT:
	RETVAL

