#!/usr/bin/perl -w

=head1 NAME

B<POWER::Tree::AnyValue> -  POWER::Tree   
   .

=head1 SYNOPSIS

 $tree  = POWER::Tree::AnyValue->root("the root");
 $tree->delimiter("/");
 $p1    = $tree->add_value("/",    "point_1") or print $tree->err,"\n";
 $p1_11 = $tree->add_value($p1,    "point_1.1");
          $tree->add_value($p1_11, "point_1.1.1");
 $p2    = $tree->add_value("/",    "point_2");
	  $tree->add_value($p2,    "point_2.1");
 @list  = @{ $tree->tree2list() };
 print map { join(" ", @$_)."\n" } @list;

 (    ):

 =- the root 			/
     - point_1 			/1
     | v- point_1.1 		/1/2
     |    vo point_1.1.1 	/1/2/3
    v- point_2 			/4
       vo point_2.1 		/4/5

=head1 DESCRIPTION

       .
     ,     ,   
POWER::Tree.         
 .

=head1 IDEA

  -,   POWER::Tree,    : 
       ,   
     : VALUE -   , 
 ID -     .    ID,    
 .  ID   
     ID.

 $node1 = [ $node_1_value, $node_1_1, $node_1_2, ..., $node_1_N ];
 $node1_value = { VALUE => $user_scalar, ID => 555 };

:     (add_node)  (del_node) 
 ,    ID,     
  add_node  ;     
 ,      POWER::Tree::AnyValue.

=head1 METHODS

   POWER::Tree :    ,
   ,    ,
         .

    POWER::Tree.

/ :

=over 4

=cut

package POWER::Tree::AnyValue;
use POWER::Tree;
use strict;
use vars qw(@ISA $VERSION);
$VERSION = "1.00" ;
@ISA = qw(POWER::Tree);

=item B<root>( $value )

   (/).
        .
   $value      $value.

=cut

sub root {
    my ($self, $value) = @_;
    my $tree = $self->SUPER::root();
    $tree->value(undef, $value);
    return $tree;
}

sub path {
    my ($self, $value) = @_;
    return $self->delimiter . $value->{ID};
}
sub add_value {
    my ($self, $path, $value, $pos) = @_ ;
    my $ID = ++$self->{ID};
    $self->SUPER::add_value($path, {ID=>$ID,VALUE=>$value}, $pos);
}
sub value {
    my ($self, $path) = (shift, shift);
    my $node = $self->find_node($path);
    $node->[0]{VALUE} = shift if @_;
    return $node->[0]{VALUE};
}

=back

=head1 AUTHOR

 Alex Efros <powerman@consultant.com>

=cut
